#apply the refraction correction to the mask results

import numpy as np
from astropy.table import Table,Column
def refraction_calc(ra_center,dec_center):
    rph = np.pi*15./180 #radians per hour
    rpd = np.pi/180. #radians per degree
    dpr = 1/rpd #radians per degree
    rpam = rpd/60. #radiams per arcmin
    H = 0 * rph  #chosen hour angle
#=================================================================================================================================
    #generate grid of RA and Dec based on mask center position for the whole field of view of LRIS
    #field of view of LRIS is 6×7.8 arcmin
    delta =  4 * rpam

    ra_range = np.linspace(ra_center-delta,ra_center+delta, num = 100)
    dec_range = np.linspace(dec_center-delta,dec_center+delta, num = 100)
    ra_values = []
    dec_values = []
    ra_offsets = []
    dec_offsets = []
    offsets = []
#=================================================================================================================================
    #create the grid of ra and dec with the ranges there
    ra_grid = np.zeros(shape=(len(ra_range),len(dec_range)))
    dec_grid = np.zeros(shape=(len(ra_range),len(dec_range)))
    for i in range(len(ra_grid)):
        for j in range(len(dec_grid)):
            ra_grid[i:j]=ra_range[i]
            dec_grid[i:j]=dec_range[j]
#=================================================================================================================================
    #first calculate hour angle for the non center points
    HA = H + ra_center-ra_range

    #call function for the non center points
    #call function for the center points

    #set up the function that calculates the refraction

    def rad_refraction(ra,dec,HA):

        rph = np.pi*15./180 #radians per hour
        rpd = np.pi/180. #radians per degree
        dpr = 1/rpd #degrees per radian
        rpam = rpd/60. #radians per arcmin
        rpas = rpd/3600. #radians per arcsec
        pres = 486.  #atmos pressure
        wave = 6000./10000  #wavelength
        lat = 19.828 * rpd #keck latitude
        temp = 0

    #calculate alt and zenith distance
        sina = np.sin(lat)*np.sin(dec)+np.cos(lat)*np.cos(dec)*np.cos(HA)
        cosz = sina
        sinz = np.sqrt(1.-sina**2)
        #print(sinz)
        tanz = sinz/cosz


    #calculate parallactic angle
        if sinz != 0:
            sinq = np.cos(lat)*np.sin(HA)/sinz
            cosq = (np.cos(np.pi/2-lat) - cosz *np.cos(np.pi/2-dec))/(sinz*np.sin(np.pi/2-dec))

        else:
            sinq = 0
            cosq = 0

    #calculate refractive index
        wavers = 1/(wave**2)
        N = 64.328 + (29498.1/(146.-wavers)) + (255.4/(41.-wavers))
        N = (1*10**-6)*N


    #calculate temp and pressure correction
        tcorr = 1+0.003661*temp
        num = 720.88*tcorr
        N = N * ((pres * (1.+((1.049-0.0157*temp) * 10**-6 * pres))) / num)#N*((pres*(1.+(1.049-((0.0157*temp)*(1*10**-6)*pres))))/num)

    #calculate refraction
        R = N * 206265 * tanz

    #calculate correction to RA and Dec
        DA = R * sinq * rpas/np.cos(dec)
        DD = R * cosq * rpas
        return(ra+DA,dec+DD)

    aparent_ra = np.zeros(shape=(len(ra_range),len(dec_range)))
    aparent_dec = np.zeros(shape=(len(ra_range),len(dec_range)))

    for i in range(len(ra_range)):
        for j in range(len(dec_range)):
            temp = rad_refraction(ra_range[i],dec_range[j],HA[i])
            aparent_ra = temp[0]*dpr
            ra_values.append(aparent_ra)
            aparent_dec = temp[1]*dpr
            dec_values.append(aparent_dec)
            ra_offset = (aparent_ra-ra_range[i]*dpr)
            ra_offsets.append(ra_offset)
            dec_offset = (aparent_dec-dec_range[j]*dpr)
            dec_offsets.append(dec_offset)
            offset = np.sqrt(ra_offset**2+dec_offset**2)
            offsets.append(offset)
            
    #create an astropy table that I can then use as reference in the main code
    refraction_results= Table()
    ra_aparent = Column(ra_values, name='Aparent RA')
    dec_aparent = Column(dec_values, name='Aparent Dec')
    raoffsets = Column(ra_offsets, name='RA Offset')
    decoffsets = Column(dec_offsets, name='Dec Offset')

    refraction_results.add_column(ra_aparent)
    refraction_results.add_column(dec_aparent)
    refraction_results.add_column(raoffsets)
    refraction_results.add_column(decoffsets)

    return(refraction_results)



#code to query GAIA(or PanSTARRS or Custom) database for closest object to calculated coordinates

#import packages
import numpy as np
from astropy.coordinates import SkyCoord, ICRS, Galactic, FK4, FK5
import astropy.units as u
from astroquery.gaia import Gaia
from astroquery.mast import Catalogs
from astropy.table import Table,Column

def get_shift(data,theta,catalog_keyword,output_file,ref_system,racenter,deccenter,filename,maskbluID,adcuse):
    #Set up arrays and columns to hold results
    rpd = np.pi/180.  #radians per degree
    ra_shifts=[]
    dec_shifts = []
    ra_centers = []
    dec_centers = []
    x_centers = []
    y_centers = []
    catalog_obj_ra = []
    catalog_obj_dec = []
    objects_ra = []
    objects_dec = []
    data.add_column(0.00000000000,name='RA_Center')
    data.add_column(0.00000000000,name='Dec_Center')
    data.add_column(0.00000000000,name='X_Center')
    data.add_column(0.00000000000,name='Y_Center')
    data.add_column(0.00000000000,name='Cat_Obj_RA')
    data.add_column(0.00000000000,name='Cat_Obj_Dec')

    #Step through each slit and calcuate the center of each one by averaging
    i=0
    k =0
    print('--------------Querying Catalog-------------')
    while i <= len(data)-4:
        #Load in the corners for a given slit in both RA,Dec and x,y and calculate the ratio of the sides
        ra1,ra2,ra3,ra4 = data['Calc_RA'][i],data['Calc_RA'][i+1],data['Calc_RA'][i+2],data['Calc_RA'][i+3]
        dec1,dec2,dec3,dec4 = data['Calc_Dec'][i],data['Calc_Dec'][i+1],data['Calc_Dec'][i+2],data['Calc_Dec'][i+3]
        side1 = 3600*np.sqrt(((ra2-ra1)*np.cos(dec1*np.pi/180))**2+(dec1-dec2)**2)
        side2 = 3600*np.sqrt(((ra2-ra3)*np.cos(dec2*np.pi/180))**2+(dec3-dec2)**2)
        ratio = side1/side2
        ra_arr=[ra1,ra2,ra3,ra4]
        dec_arr=[dec1,dec2,dec3,dec4]
        ra_avg = (np.max(ra_arr)+np.min(ra_arr))/2.
        dec_avg = (np.max(dec_arr)+np.min(dec_arr))/2.
        xpos = [data['X'][i],data['X'][i+1],data['X'][i+2],data['X'][i+3]]
        ypos = [data['Y'][i],data['Y'][i+1],data['Y'][i+2],data['Y'][i+3]]
        x_avg = (np.max(xpos)+np.min(xpos))/2.
        y_avg = (np.max(ypos)+np.min(ypos))/2.
        data['RA_Center'][i],data['RA_Center'][i+1],data['RA_Center'][i+2],data['RA_Center'][i+3]=ra_avg,ra_avg,ra_avg,ra_avg
        data['Dec_Center'][i],data['Dec_Center'][i+1],data['Dec_Center'][i+2],data['Dec_Center'][i+3]=dec_avg,dec_avg,dec_avg,dec_avg
        data['X_Center'][i],data['X_Center'][i+1],data['X_Center'][i+2],data['X_Center'][i+3]=x_avg,x_avg,x_avg,x_avg
        data['Y_Center'][i],data['Y_Center'][i+1],data['Y_Center'][i+2],data['Y_Center'][i+3]=y_avg,y_avg,y_avg,y_avg
#=================================================================================================================================
        #decide if current slit is an alignment box based on the ratio of the lengths of the sides of the slits
        if 0.95 < ratio < 1.05:
            k=k+1
            ra_centers.append(ra_avg)
            dec_centers.append(dec_avg)
            x_centers.append(x_avg)
            y_centers.append(y_avg)
            print('ra,dec of box =',ra_avg,dec_avg)

#=================================================================================================================================
            #call astroquery with the chosen catalog to gather list of objects in and near the alignment boxes

            coord = SkyCoord(ra=ra_avg, dec=dec_avg, unit=(u.deg,u.deg),frame=ref_system)
            if catalog_keyword == 'gaia':
                width = 200*u.arcsec
                height = 200*u.arcsec
                obj = Gaia.query_object_async(coordinate=coord, width=width,height=height)
                mag_cutoff = (obj['phot_g_mean_flux']>18.)
                obj = obj[mag_cutoff]

                objects_ra.append(obj['ra'])
                objects_dec.append(obj['dec'])

                objects_ra.append(obj['ra'])
                objects_dec.append(obj['dec'])

                #find the object with the closest position
                diff = []
                for j in range(len(obj)):
                    diff.append(np.sqrt(((obj['ra'][j]-ra_avg)*np.cos(dec_avg*np.pi/180.))**2+(obj['dec'][j]-dec_avg)**2))
                min_diff = np.argmin(diff)

                #save catalog object to array
                catalog_obj_ra.append(obj[min_diff]['ra'])
                catalog_obj_dec.append(obj[min_diff]['dec'])

                data['Cat_Obj_RA'][i],data['Cat_Obj_RA'][i+1],data['Cat_Obj_RA'][i+2],data['Cat_Obj_RA'][i+3] = obj[min_diff]['ra'],obj[min_diff]['ra'],obj[min_diff]['ra'],obj[min_diff]['ra']
                data['Cat_Obj_Dec'][i],data['Cat_Obj_Dec'][i+1],data['Cat_Obj_Dec'][i+2],data['Cat_Obj_Dec'][i+3] = obj[min_diff]['dec'],obj[min_diff]['dec'],obj[min_diff]['dec'],obj[min_diff]['dec']

                print('GAIA object located at:',obj[min_diff]['ra'],obj[min_diff]['dec'])
                ra_shifts.append((ra_avg-obj['ra'][min_diff])*np.cos(dec_avg*np.pi/180.))
                dec_shifts.append(dec_avg-obj['dec'][min_diff])

            if catalog_keyword == 'panstarrs':
                obj = Catalogs.query_region(coord, radius=15*u.arcsec, catalog='Panstarrs', data_release='dr2')
               # mag_cutoff = (obj['gMeanKronMag']>18.)
                #obj = obj[mag_cutoff]

                objects_ra.append(obj['raMean'])
                objects_dec.append(obj['decMean'])


                #find the object with the closest position
                diff = []
                for j in range(len(obj)):
                    diff.append(np.sqrt(((obj['raMean'][j]-ra_avg)*np.cos(dec_avg*np.pi/180.))**2+(obj['decMean'][j]-dec_avg)**2))
                min_diff = np.argmin(diff)


                #save catalog object to array
                catalog_obj_ra.append(obj[min_diff]['raMean'])
                catalog_obj_dec.append(obj[min_diff]['decMean'])

                data['Cat_Obj_RA'][i],data['Cat_Obj_RA'][i+1],data['Cat_Obj_RA'][i+2],data['Cat_Obj_RA'][i+3] = obj[min_diff]['raMean'],obj[min_diff]['raMean'],obj[min_diff]['raMean'],obj[min_diff]['raMean']
                data['Cat_Obj_Dec'][i],data['Cat_Obj_Dec'][i+1],data['Cat_Obj_Dec'][i+2],data['Cat_Obj_Dec'][i+3] = obj[min_diff]['decMean'],obj[min_diff]['decMean'],obj[min_diff]['decMean'],obj[min_diff]['decMean']

                print('PANSTARRS object located at:',obj[min_diff]['raMean'],obj[min_diff]['decMean'])
                ra_shifts.append((ra_avg-obj['raMean'][min_diff])*np.cos(dec_avg*np.pi/180.))
                dec_shifts.append(dec_avg-obj['decMean'][min_diff])

            if catalog_keyword == 'custom':
                obj = Table.read(filename)
                objects_ra.append(obj['ra'])
                objects_dec.append(obj['dec'])

                #find the object with the closest position
                diff = []
                for j in range(len(obj)):
                    diff.append(np.sqrt(((obj['ra'][j]-ra_avg)*np.cos(dec_avg*np.pi/180.))**2+(obj['dec'][j]-dec_avg)**2))
                min_diff = np.argmin(diff)

                #save catalog object to array
                catalog_obj_ra.append(obj[min_diff]['ra'])
                catalog_obj_dec.append(obj[min_diff]['dec'])

                data['Cat_Obj_RA'][i],data['Cat_Obj_RA'][i+1],data['Cat_Obj_RA'][i+2],data['Cat_Obj_RA'][i+3] = obj[min_diff]['ra'],obj[min_diff]['ra'],obj[min_diff]['ra'],obj[min_diff]['ra']
                data['Cat_Obj_Dec'][i],data['Cat_Obj_Dec'][i+1],data['Cat_Obj_Dec'][i+2],data['Cat_Obj_Dec'][i+3] = obj[min_diff]['dec'],obj[min_diff]['dec'],obj[min_diff]['dec'],obj[min_diff]['dec']

                print('Custom object located at:',obj[min_diff]['ra'],obj[min_diff]['dec'])
                ra_shifts.append((ra_avg-obj['ra'][min_diff])*np.cos(dec_avg*np.pi/180.))
                dec_shifts.append(dec_avg-obj['dec'][min_diff])

            i=i+4
        else:
            i=i+4
#=================================================================================================================================
#Because we are not sure if the catalog object matched with each box is the correct one, we determine which shifts are likely due to correct objects
#being matched to the boxes (ie. "good" shifts). This is done by determining whether the shift array contains both "small" and "large" shift values (as
#defined by cutoff values). If this is the case, the outliers are found by removing the small/large values by determining which the shift array has more
#of and removing the small/large values that the array has fewer of.

    good_ra_shifts=[]
    good_dec_shifts=[]
    removed = 0
    large = 0
    small = 0
    if (np.min(np.abs(ra_shifts))*3600 < 1.2 and np.max(np.abs(ra_shifts))*3600 > 5.0) or (np.min(np.abs(dec_shifts))*3600 < 1.2 and np.max(np.abs(dec_shifts))*3600 > 5.0):
        print('Found Outliers...')
        for h in range(len(ra_shifts)):
            if np.abs(ra_shifts[h])*3600 < 1.2 and np.abs(dec_shifts[h])*3600 < 1.2:
                small = small +1
            else:
                large = large+1
        if small >= large:
            for n in range(len(ra_shifts)):
                if np.abs(ra_shifts[n])*3600 > 5. or np.abs(dec_shifts[n])*3600 > 5.0:
                    removed = removed +1
                    n=n+1
                else:
                    good_ra_shifts.append(ra_shifts[n])
                    good_dec_shifts.append(dec_shifts[n])
        if large > small:
            for n in range(len(ra_shifts)):
                if np.abs(ra_shifts[n])*3600 < 5. or np.abs(dec_shifts[n])*3600 < 5.0:
                    removed = removed +1
                    n=n+1
                else:
                    good_ra_shifts.append(ra_shifts[n])
                    good_dec_shifts.append(dec_shifts[n])
    else:
        good_ra_shifts = ra_shifts
        good_dec_shifts = dec_shifts

    print(small,'small shifts')
    print(removed,'elements removed from shift arrays')
#=================================================================================================================================
#Calculate the final shifts to apply to the slit positions by averaging the individual "good" shifts from the boxes.
    dec_shift_final = np.average(good_dec_shifts)
    ra_shift_final = np.average(good_ra_shifts)/np.cos(data['Calc_Dec'][0]*np.pi/180)
    final_shifts = np.zeros(len(good_ra_shifts))
    for g in range(len(final_shifts)):
        final_shifts[g] = np.sqrt((good_ra_shifts[g]/np.cos(data['Calc_Dec'][0]*np.pi/180))**2+(good_dec_shifts[g]**2))*3600
    total_shift_final = np.sqrt(ra_shift_final**2+dec_shift_final**2)*3600
    total_shift_dispersion =np.sqrt(np.sum((final_shifts-total_shift_final)**2)/(len(final_shifts)-1))

#Determine whether or not to apply a shift. If the total dispersion value for the shift is too large, or is above 0.7 times the shift values
#do not apply a shift.
    if total_shift_dispersion >=2.0 or np.isnan(total_shift_final) == True:
        ra_shift_final = np.float64(0.0)
        dec_shift_final = np.float64(0.0)
        print('---------No Systematic Shift Found. No Shift Applied---------')
    if 0.70 < total_shift_dispersion/total_shift_final:
        ra_shift_final = np.float64(0.0)
        dec_shift_final = np.float64(0.0)
        print('---------No Systematic Shift Found. No Shift Applied---------')

    ra_shifted_centers = ra_centers-ra_shift_final
    dec_shifted_centers = dec_centers-dec_shift_final
    print('Final Shift:',str(round(total_shift_final,3))+'+/-'+str(round(total_shift_dispersion,3)))


    ra_shifted_centers = ra_centers-ra_shift_final
    dec_shifted_centers = dec_centers-dec_shift_final

#=================================================================================================================================
#Calculate the difference between the shifted centers and the catalog objects
    after_shifts_total = np.zeros(len(ra_centers))
    before_shifts_total = np.zeros(len(ra_centers))
    after_ra_shifts = np.zeros(len(ra_centers))
    before_ra_shifts = ra_shifts
    for g in range(len(after_shifts_total)):
        before_ra_diff = 3600*(ra_centers[g] - catalog_obj_ra[g])*np.cos(data['Calc_Dec'][0]*np.pi/180)
        before_dec_diff = 3600*(dec_centers[g] - catalog_obj_dec[g])
        ra_diff = 3600*(ra_shifted_centers[g] - catalog_obj_ra[g])*np.cos(data['Calc_Dec'][0]*np.pi/180)
        after_ra_shifts[g]=ra_diff
        before_ra_shifts[g] = 3600.*ra_shifts[g]
        dec_diff = 3600*(dec_shifted_centers[g] - catalog_obj_dec[g])
        before_shifts_total[g] = np.sqrt((before_ra_diff)**2+(before_dec_diff)**2)
        after_shifts_total[g] = np.sqrt((ra_diff)**2+(dec_diff)**2)
    print('Before Shifts:',before_shifts_total)
    print('After Shifts:',after_shifts_total)
    print('Before RA Shifts:',before_ra_shifts)
    print('After RA Shifts:',after_ra_shifts)
#=================================================================================================================================
#Shift the slit positions
    data['Calc_RA'] = data['Calc_RA']-ra_shift_final
    data['Calc_Dec'] = data['Calc_Dec']-dec_shift_final
    data['RA_Center'] = data['RA_Center']-ra_shift_final
    data['Dec_Center'] = data['Dec_Center']-dec_shift_final

    return(data,x_centers,y_centers,ra_shifted_centers,dec_shifted_centers,catalog_obj_ra,catalog_obj_dec,objects_ra,objects_dec)



#calculate the final correction autoslit makes to the x,y positions for the milling maching output file.
#This doesn't depend on WCS position, so I should just calculate a single table that the code reads in

import numpy as np
from astropy.table import Table,Column
def astrometry_calc(ra_center,dec_center):

    #create a grid of values to create the points for which the correction will be calculated
    x_range = np.linspace(174,436, num = 500)
    y_range = np.linspace(-174,174, num = 500)

    #set up arrays to hold results later
    x_values = []
    y_values = []
    x_offsets = []
    y_offsets = []
    offsets = []
#=================================================================================================================================
    #create the routine for the astrometry correction
    def astrometry_correction_calc(XIN,YIN):
        #set some constants
        CCD_SIDE = 2048.0
        CCD_SCALE = 0.15767
        X_CENTER = 305.0  #center of the mask in x
        Y_CENTER = 0.0    #center of the mask in y
        #set the fits values from autoslit that are used in the correction
        A=[2.0,  0.99476227,0.00728125,0.45412825E-6, -0.40955557E-5,0.96690643E-6]
        B=[1.0, -0.00001856, 0.99906324 ]

        #Convert mm to CCD pixels. (Approximate, but as long as you
        #use exactly the same formula for inversion, that is OK.)
        XCCD = CCD_SIDE / 2.0 + (X_CENTER - XIN) / CCD_SCALE
        YCCD = CCD_SIDE / 2.0 - (Y_CENTER - YIN) / CCD_SCALE

        XCCD_OUT = A[0] + A[1] * XCCD + A[2] * YCCD + A[3] * XCCD * XCCD + A[4] * YCCD * YCCD + A[5] * XCCD * YCCD
        YCCD_OUT = B[0] + B[1] * XCCD + B[2] * YCCD

        #Now convert back to mm on the slitmask.

        XOUT = X_CENTER - (XCCD_OUT - CCD_SIDE / 2.0) * CCD_SCALE
        YOUT = Y_CENTER + (YCCD_OUT - CCD_SIDE / 2.0) * CCD_SCALE

        return(XOUT,YOUT)
#=================================================================================================================================
    #set up more arrays to hold results
    corrected_x = np.zeros(shape=(len(x_range),len(y_range)))
    corrected_y = np.zeros(shape=(len(x_range),len(y_range)))

    #run through the values of the grid and put each point through the slit_astrometry calculation
    #appending results to the arrays from earlier in prep for creating an astropy table to return the results in
    for i in range(len(x_range)):
        for j in range(len(y_range)):
            temp = astrometry_correction_calc(x_range[i],y_range[j])
            corrected_x = temp[0]
            x_values.append(corrected_x)
            corrected_y= temp[1]
            y_values.append(corrected_y)
            x_offset = (corrected_x-x_range[i])
            x_offsets.append(x_offset)
            y_offset = (corrected_y-y_range[j])
            y_offsets.append(y_offset)
            offset = np.sqrt(x_offset**2+y_offset**2)
            offsets.append(offset)
#=================================================================================================================================            
    #create the astropy table and fill it in
    astrometry_results= Table()
    x_aparent = Column(x_values, name='Aparent X')
    y_aparent = Column(y_values, name='Aparent Y')
    xoffsets = Column(x_offsets, name='X Offset')
    yoffsets = Column(y_offsets, name='Y Offset')

    astrometry_results.add_column(x_aparent)
    astrometry_results.add_column(y_aparent)
    astrometry_results.add_column(xoffsets)
    astrometry_results.add_column(yoffsets)

    #return the astropy results table
    return(astrometry_results)



#Create the quick look plot of the mask results and save as a pdf

import subprocess
import os
from astropy.io import fits
from astropy.wcs import WCS,utils
from astropy.table import Table
from matplotlib import pyplot as plt
from astropy.coordinates import SkyCoord, ICRS, Galactic, FK4, FK5
import numpy as np
import astropy.units as u
from matplotlib.patches import Polygon as Pgon
from astropy.visualization import PercentileInterval, ImageNormalize
from glob import glob
from matplotlib.gridspec import GridSpec

#Function to create the quick look plots of teh mask results
def create_err_plot(data,catalog_obj_ra,catalog_obj_dec,objects_ra,objects_dec,output_file):

    #Calculate the center of the mask to provide PanSTAMPS to download the image for the mask field  (should I just send over the actual center coords)
    x_center = np.average(data['Calc_RA'])
    y_center = np.average(data['Calc_Dec'])

    #Download the PanSTARRS field image for the mask location

    panstampscommand = 'panstamps --width=15 --filter=g stack '+str(x_center)+' '+str(y_center)
    process2 = subprocess.Popen(panstampscommand.split(), stdout=subprocess.PIPE)
    output, error = process2.communicate()
    process2.wait()

    #Open the field image. Just picks the first image on the list generated by glob
    field_file_path = str(x_center)+'*/''*.fits'
    all_field_files = glob(field_file_path)
    hdu = fits.open(all_field_files[0])[0]
#=================================================================================================================================
    #Set up the grid for the plot panels using GridSpec
    wcs = WCS(hdu.header)

    fig = plt.figure(constrained_layout=True,figsize=(7,4))
    gs = GridSpec(3, 3, figure=fig,width_ratios = [3,1,1],height_ratios=[1,1,1])
    ax1 = fig.add_subplot(gs[:, :-2],projection=wcs)
    ax2 = fig.add_subplot(gs[0, 1],projection=wcs)
    ax3 = fig.add_subplot(gs[1, 1],projection=wcs)
    ax4 = fig.add_subplot(gs[2, 1],projection=wcs)
    ax5 = fig.add_subplot(gs[0, 2],projection=wcs)
    ax6 = fig.add_subplot(gs[1, 2],projection=wcs)
    ax7 = fig.add_subplot(gs[2, 2],projection=wcs)

    #For clarity of the plot, set the ticks of the zoom in panels on the alignment boxes to be invisible
    lon2 = ax2.coords[0]
    lat2 = ax2.coords[1]
    lon2.set_ticks_visible(False)
    lat2.set_ticklabel_visible(False)
    lat2.set_ticks_visible(False)
    lon2.set_ticklabel_visible(False)
    lon3 = ax3.coords[0]
    lat3 = ax3.coords[1]
    lon3.set_ticks_visible(False)
    lat3.set_ticklabel_visible(False)
    lat3.set_ticks_visible(False)
    lon3.set_ticklabel_visible(False)
    lon4 = ax4.coords[0]
    lat4 = ax4.coords[1]
    lon4.set_ticks_visible(False)
    lat4.set_ticklabel_visible(False)
    lat4.set_ticks_visible(False)
    lon4.set_ticklabel_visible(False)
    lon5 = ax5.coords[0]
    lat5 = ax5.coords[1]
    lon5.set_ticks_visible(False)
    lat5.set_ticklabel_visible(False)
    lat5.set_ticks_visible(False)
    lon5.set_ticklabel_visible(False)
    lon6 = ax6.coords[0]
    lat6 = ax6.coords[1]
    lon6.set_ticks_visible(False)
    lat6.set_ticklabel_visible(False)
    lat6.set_ticks_visible(False)
    lon6.set_ticklabel_visible(False)
    lon7 = ax7.coords[0]
    lat7 = ax7.coords[1]
    lon7.set_ticks_visible(False)
    lat7.set_ticklabel_visible(False)
    lat7.set_ticks_visible(False)
    lon7.set_ticklabel_visible(False)

#=================================================================================================================================
    #Create the main plot. This is the full mask view.
    norm = ImageNormalize(hdu.data, interval=PercentileInterval(99))
    norm2 = ImageNormalize(hdu.data, interval=PercentileInterval(99))
    i=0
    avg_y_pos = []
    box_numbers = []

    #order the slits by y positions
    while i in range(len(data)):
        avg_y_pos.append(np.average(data['Y'][i:i+4]))
        i = i+4
    avg_y_pos = np.sort(avg_y_pos)
    i=0
    while i in range(int(len(data))):
        vertices = np.zeros(shape=(4,2))
        averages = np.zeros(shape=(2,2))
        vertices[0:4,0] = data['Calc_RA'][i:i+4]
        vertices[0:4,1] = data['Calc_Dec'][i:i+4]
        averages[0,0]=np.average((vertices[1,0],vertices[2,0]))
        averages[1,0] = np.average((vertices[0,0],vertices[3,0]))
        averages[0,1]=np.average((vertices[1,1],vertices[2,1]))
        averages[1,1]=np.average((vertices[0,1],vertices[3,1]))
        ra_center = np.average(vertices[:,0])
        dec_center = np.average(vertices[:,1])
        y_avg = np.average(data['Y'][i:i+4])
        for k in range(len(avg_y_pos)):
            if y_avg == avg_y_pos[k]:
                ind = k+1
                ra1,ra2,ra3,ra4 = data['Calc_RA'][i],data['Calc_RA'][i+1],data['Calc_RA'][i+2],data['Calc_RA'][i+3]
                dec1,dec2,dec3,dec4 = data['Calc_Dec'][i],data['Calc_Dec'][i+1],data['Calc_Dec'][i+2],data['Calc_Dec'][i+3]
                side1 = 3600*np.sqrt(((ra2-ra1)*np.cos(dec1*np.pi/180))**2+(dec1-dec2)**2)
                side2 = 3600*np.sqrt(((ra2-ra3)*np.cos(dec2*np.pi/180))**2+(dec3-dec2)**2)
                ratio = side1/side2
                if 0.9 < ratio < 1.1:
                    box_numbers.append(ind)
            k=k+1
        pixelcoords = []
        for l in range(0,4):
            temp = SkyCoord(vertices[l,0],vertices[l,1], unit=(u.deg,u.deg),frame='fk5')
            temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
            pixelcoords.append(temp2)
        r = Pgon(pixelcoords, closed = True, edgecolor='green', facecolor='none',alpha=0.6,zorder=4)
        ax1.add_patch(r)
        ax1.text(ra_center,dec_center,str(ind),color='blue',transform=ax1.get_transform('fk5'))
        i = i+4


    im = ax1.imshow(hdu.data,origin='lower',zorder=0,cmap='gray_r',norm=norm)
    temp1a = SkyCoord(np.min(data['Calc_RA'])-50/3600.,np.min(data['Calc_Dec'])-50/3600.,unit=(u.deg,u.deg))
    temp1b = SkyCoord(np.max(data['Calc_RA'])+50/3600.,np.max(data['Calc_Dec'])+50/3600.,unit=(u.deg,u.deg))
    pixels1a=utils.skycoord_to_pixel(temp1a, wcs=wcs, origin=0, mode='all')
    pixels1b=utils.skycoord_to_pixel(temp1b, wcs=wcs, origin=0, mode='all')
    ax1.set_xlim([pixels1a[0],pixels1b[0]])
    ax1.set_ylim([pixels1a[1],pixels1b[1]])
    ax1.invert_xaxis()
#=================================================================================================================================
    #Find the first six boxes and plot the cutout panels in the quick look plot.
    i=0
    boxes=0
    while i <= len(data)-4 and boxes < 6:
        ra1,ra2,ra3,ra4 = data['Calc_RA'][i],data['Calc_RA'][i+1],data['Calc_RA'][i+2],data['Calc_RA'][i+3]
        dec1,dec2,dec3,dec4 = data['Calc_Dec'][i],data['Calc_Dec'][i+1],data['Calc_Dec'][i+2],data['Calc_Dec'][i+3]
        side1 = 3600*np.sqrt(((ra2-ra1)*np.cos(dec1*np.pi/180))**2+(dec1-dec2)**2)
        side2 = 3600*np.sqrt(((ra2-ra3)*np.cos(dec2*np.pi/180))**2+(dec3-dec2)**2)
        ratio = side1/side2
        if 0.9 < ratio < 1.1:
            boxes = boxes+1
            vertices = np.zeros(shape=(4,2))
            vertices[0:4,0] = data['Calc_RA'][i:i+4]
            vertices[0:4,1] = data['Calc_Dec'][i:i+4]
            ra_buffer = (5/(3600.*np.cos(vertices[0,1]*np.pi/180.)))
            dec_buffer = 5/3600.
            if boxes == 1:
                pixelcoords = []
                for l in range(len(vertices)):
                    temp = SkyCoord(vertices[l,0],vertices[l,1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoords.append(temp2)
                r = Pgon(pixelcoords, closed = True, edgecolor='green', facecolor='none',alpha=0.6,zorder=4)
                ax2.add_patch(r)
                im2 = ax2.imshow(hdu.data,origin='lower',zorder=0,cmap='gray_r',norm=norm2)
                temp1a = SkyCoord(np.min(vertices[0:4,0])-ra_buffer,np.min(vertices[0:4,1])-dec_buffer,unit=(u.deg,u.deg))
                temp1b = SkyCoord(np.max(vertices[0:4,0])+ra_buffer,np.max(vertices[0:4,1])+dec_buffer,unit=(u.deg,u.deg))
                pixels1a=utils.skycoord_to_pixel(temp1a, wcs=wcs, origin=1, mode='all')
                pixels1b=utils.skycoord_to_pixel(temp1b, wcs=wcs, origin=1, mode='all')
                temp1c = SkyCoord(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],unit=(u.deg,u.deg))
                pixels1c = utils.skycoord_to_pixel(temp1c, wcs=wcs, origin=1, mode='all')
                ax2.scatter(pixels1c[0],pixels1c[1],zorder=5,color='yellow')#,transform=ax2.get_transform('fk5'))
                #ax2.scatter(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],zorder=5,color='yellow',transform=ax2.get_transform('fk5'))
                pixelcoordsra= []
                pixelcoordsdec= []
                for l in range(len(objects_ra[boxes-1])):
                    temp = SkyCoord(objects_ra[boxes-1],objects_dec[boxes-1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoordsra.append(temp2[0])
                    pixelcoordsdec.append(temp2[1])
                ax2.scatter(pixelcoordsra,pixelcoordsdec,zorder=4,color='orange')
                ax2.text(np.min(vertices[0:4,0])-ra_buffer/3,np.min(vertices[0:4,1])-dec_buffer/3,str(box_numbers[boxes-1]),color='blue',transform=ax2.get_transform('fk5'))
                ax2.set_xlim([pixels1a[0],pixels1b[0]])
                ax2.set_ylim([pixels1a[1],pixels1b[1]])
                ax2.invert_xaxis()
            if boxes == 2:
                pixelcoords = []
                for l in range(len(vertices)):
                    temp = SkyCoord(vertices[l,0],vertices[l,1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoords.append(temp2)
                r = Pgon(pixelcoords, closed = True, edgecolor='green', facecolor='none',alpha=0.6,zorder=4)
                ax3.add_patch(r)
                im3 = ax3.imshow(hdu.data,origin='lower',zorder=0,cmap='gray_r',norm=norm2)
                temp1a = SkyCoord(np.min(vertices[0:4,0])-ra_buffer,np.min(vertices[0:4,1])-dec_buffer,unit=(u.deg,u.deg))
                temp1b = SkyCoord(np.max(vertices[0:4,0])+ra_buffer,np.max(vertices[0:4,1])+dec_buffer,unit=(u.deg,u.deg))
                pixels1a=utils.skycoord_to_pixel(temp1a, wcs=wcs, origin=1, mode='all')
                pixels1b=utils.skycoord_to_pixel(temp1b, wcs=wcs, origin=1, mode='all')
                temp1c = SkyCoord(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],unit=(u.deg,u.deg))
                pixels1c = utils.skycoord_to_pixel(temp1c, wcs=wcs, origin=1, mode='all')
                ax3.scatter(pixels1c[0],pixels1c[1],zorder=5,color='yellow')
                #ax3.scatter(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],zorder=5,color='yellow',transform=ax3.get_transform('world'))
                pixelcoordsra= []
                pixelcoordsdec= []
                for l in range(len(objects_ra[boxes-1])):
                    temp = SkyCoord(objects_ra[boxes-1],objects_dec[boxes-1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoordsra.append(temp2[0])
                    pixelcoordsdec.append(temp2[1])
                ax3.scatter(pixelcoordsra,pixelcoordsdec,zorder=4,color='orange')
                #ax3.scatter(objects_ra[boxes-1],objects_dec[boxes-1],zorder=4,color='purple',transform=ax3.get_transform('fk5'))
                ax3.text(np.min(vertices[0:4,0])-ra_buffer/3,np.min(vertices[0:4,1])-dec_buffer/3,str(box_numbers[boxes-1]),color='blue',transform=ax3.get_transform('fk5'))
                ax3.set_xlim([pixels1a[0],pixels1b[0]])
                ax3.set_ylim([pixels1a[1],pixels1b[1]])
                ax3.invert_xaxis()
            if boxes == 3:
                pixelcoords = []
                for l in range(len(vertices)):
                    temp = SkyCoord(vertices[l,0],vertices[l,1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoords.append(temp2)
                r = Pgon(pixelcoords, closed = True, edgecolor='green', facecolor='none',alpha=0.6,zorder=4)
                ax4.add_patch(r)
                im4 = ax4.imshow(hdu.data,origin='lower',zorder=0,cmap='gray_r',norm=norm2)
                temp1a = SkyCoord(np.min(vertices[0:4,0])-ra_buffer,np.min(vertices[0:4,1])-dec_buffer,unit=(u.deg,u.deg))
                temp1b = SkyCoord(np.max(vertices[0:4,0])+ra_buffer,np.max(vertices[0:4,1])+dec_buffer,unit=(u.deg,u.deg))
                pixels1a=utils.skycoord_to_pixel(temp1a, wcs=wcs, origin=1, mode='all')
                pixels1b=utils.skycoord_to_pixel(temp1b, wcs=wcs, origin=1, mode='all')
                temp1c = SkyCoord(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],unit=(u.deg,u.deg))
                pixels1c = utils.skycoord_to_pixel(temp1c, wcs=wcs, origin=1, mode='all')
                ax4.scatter(pixels1c[0],pixels1c[1],zorder=5,color='yellow')
                #ax4.scatter(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],zorder=5,color='yellow',transform=ax4.get_transform('world'))
                pixelcoordsra= []
                pixelcoordsdec= []
                for l in range(len(objects_ra[boxes-1])):
                    temp = SkyCoord(objects_ra[boxes-1],objects_dec[boxes-1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoordsra.append(temp2[0])
                    pixelcoordsdec.append(temp2[1])
                ax4.scatter(pixelcoordsra,pixelcoordsdec,zorder=4,color='orange')
              #  ax4.scatter(objects_ra[boxes-1],objects_dec[boxes-1],zorder=4,color='purple',transform=ax4.get_transform('fk5'))
                ax4.text(np.min(vertices[0:4,0])-ra_buffer/3,np.min(vertices[0:4,1])-dec_buffer/3,str(box_numbers[boxes-1]),color='blue',transform=ax4.get_transform('fk5'))
                ax4.set_xlim([pixels1a[0],pixels1b[0]])
                ax4.set_ylim([pixels1a[1],pixels1b[1]])
                ax4.invert_xaxis()
            if boxes == 4:
                pixelcoords = []
                for l in range(len(vertices)):
                    temp = SkyCoord(vertices[l,0],vertices[l,1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoords.append(temp2)
                r = Pgon(pixelcoords, closed = True, edgecolor='green', facecolor='none',alpha=0.6,zorder=4)
                ax5.add_patch(r)
                im5 = ax5.imshow(hdu.data,origin='lower',zorder=0,cmap='gray_r',norm=norm2)
                temp1a = SkyCoord(np.min(vertices[0:4,0])-ra_buffer,np.min(vertices[0:4,1])-dec_buffer,unit=(u.deg,u.deg))
                temp1b = SkyCoord(np.max(vertices[0:4,0])+ra_buffer,np.max(vertices[0:4,1])+dec_buffer,unit=(u.deg,u.deg))
                pixels1a=utils.skycoord_to_pixel(temp1a, wcs=wcs, origin=1, mode='all')
                pixels1b=utils.skycoord_to_pixel(temp1b, wcs=wcs, origin=1, mode='all')
                temp1c = SkyCoord(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],unit=(u.deg,u.deg))
                pixels1c = utils.skycoord_to_pixel(temp1c, wcs=wcs, origin=1, mode='all')
                ax5.scatter(pixels1c[0],pixels1c[1],zorder=5,color='yellow')
                #ax5.scatter(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],zorder=5,color='yellow',transform=ax5.get_transform('world'))
                pixelcoordsra= []
                pixelcoordsdec= []
                for l in range(len(objects_ra[boxes-1])):
                    temp = SkyCoord(objects_ra[boxes-1],objects_dec[boxes-1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoordsra.append(temp2[0])
                    pixelcoordsdec.append(temp2[1])
                ax5.scatter(pixelcoordsra,pixelcoordsdec,zorder=4,color='orange')
              #  ax5.scatter(objects_ra[boxes-1],objects_dec[boxes-1],zorder=4,color='purple',transform=ax5.get_transform('fk5'))
                ax5.text(np.min(vertices[0:4,0])-ra_buffer/3,np.min(vertices[0:4,1])-dec_buffer/3,str(box_numbers[boxes-1]),color='blue',transform=ax5.get_transform('fk5'))
                ax5.set_xlim([pixels1a[0],pixels1b[0]])
                ax5.set_ylim([pixels1a[1],pixels1b[1]])
                ax5.invert_xaxis()
            if boxes == 5:
                pixelcoords = []
                for l in range(len(vertices)):
                    temp = SkyCoord(vertices[l,0],vertices[l,1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoords.append(temp2)
                r = Pgon(pixelcoords, closed = True, edgecolor='green', facecolor='none',alpha=0.6,zorder=4)
                ax6.add_patch(r)
                im6 = ax6.imshow(hdu.data,origin='lower',zorder=0,cmap='gray_r',norm=norm2)
                temp1a = SkyCoord(np.min(vertices[0:4,0])-ra_buffer,np.min(vertices[0:4,1])-dec_buffer,unit=(u.deg,u.deg))
                temp1b = SkyCoord(np.max(vertices[0:4,0])+ra_buffer,np.max(vertices[0:4,1])+dec_buffer,unit=(u.deg,u.deg))
                pixels1a=utils.skycoord_to_pixel(temp1a, wcs=wcs, origin=1, mode='all')
                pixels1b=utils.skycoord_to_pixel(temp1b, wcs=wcs, origin=1, mode='all')
                temp1c = SkyCoord(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],unit=(u.deg,u.deg))
                pixels1c = utils.skycoord_to_pixel(temp1c, wcs=wcs, origin=1, mode='all')
                ax6.scatter(pixels1c[0],pixels1c[1],zorder=5,color='yellow')
                #ax6.scatter(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],zorder=5,color='yellow',transform=ax6.get_transform('world'))
                pixelcoordsra= []
                pixelcoordsdec= []
                for l in range(len(objects_ra[boxes-1])):
                    temp = SkyCoord(objects_ra[boxes-1],objects_dec[boxes-1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoordsra.append(temp2[0])
                    pixelcoordsdec.append(temp2[1])
                ax6.scatter(pixelcoordsra,pixelcoordsdec,zorder=4,color='orange')
            #    ax6.scatter(objects_ra[boxes-1],objects_dec[boxes-1],zorder=4,color='purple',transform=ax6.get_transform('fk5'))
                ax6.text(np.min(vertices[0:4,0])-ra_buffer/3,np.min(vertices[0:4,1])-dec_buffer/3,str(box_numbers[boxes-1]),color='blue',transform=ax6.get_transform('fk5'))
                ax6.set_xlim([pixels1a[0],pixels1b[0]])
                ax6.set_ylim([pixels1a[1],pixels1b[1]])
                ax6.invert_xaxis()
            if boxes == 6:
                pixelcoords = []
                for l in range(len(vertices)):
                    temp = SkyCoord(vertices[l,0],vertices[l,1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoords.append(temp2)
                r = Pgon(pixelcoords, closed = True, edgecolor='green', facecolor='none',alpha=0.6,zorder=4)
                ax7.add_patch(r)
                im7 = ax7.imshow(hdu.data,origin='lower',zorder=0,cmap='gray_r',norm=norm2)
                temp1a = SkyCoord(np.min(vertices[0:4,0])-ra_buffer,np.min(vertices[0:4,1])-dec_buffer,unit=(u.deg,u.deg))
                temp1b = SkyCoord(np.max(vertices[0:4,0])+ra_buffer,np.max(vertices[0:4,1])+dec_buffer,unit=(u.deg,u.deg))
                pixels1a=utils.skycoord_to_pixel(temp1a, wcs=wcs, origin=1, mode='all')
                pixels1b=utils.skycoord_to_pixel(temp1b, wcs=wcs, origin=1, mode='all')
                temp1c = SkyCoord(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],unit=(u.deg,u.deg))
                pixels1c = utils.skycoord_to_pixel(temp1c, wcs=wcs, origin=1, mode='all')
                ax7.scatter(pixels1c[0],pixels1c[1],zorder=5,color='yellow')
                #ax7.scatter(catalog_obj_ra[boxes-1],catalog_obj_dec[boxes-1],zorder=5,color='yellow',transform=ax7.get_transform('world'))
                pixelcoordsra= []
                pixelcoordsdec= []
                for l in range(len(objects_ra[boxes-1])):
                    temp = SkyCoord(objects_ra[boxes-1],objects_dec[boxes-1], unit=(u.deg,u.deg),frame='fk5')
                    temp2 = utils.skycoord_to_pixel(temp, wcs=wcs, origin=1, mode='all')
                    pixelcoordsra.append(temp2[0])
                    pixelcoordsdec.append(temp2[1])
                ax7.scatter(pixelcoordsra,pixelcoordsdec,zorder=4,color='orange')
           #     ax7.scatter(objects_ra[boxes-1],objects_dec[boxes-1],zorder=4,color='purple',transform=ax7.get_transform('fk5'))
                ax7.text(np.min(vertices[0:4,0])-ra_buffer/3,np.min(vertices[0:4,1])-dec_buffer/3,str(box_numbers[boxes-1]),color='blue',transform=ax7.get_transform('fk5'))
                ax7.set_xlim([pixels1a[0],pixels1b[0]])
                ax7.set_ylim([pixels1a[1],pixels1b[1]])
                ax7.invert_xaxis()
        i = i+4
    plt.savefig(output_file+'quicklookplot.pdf',bbox_inches='tight',pad_inches=0.5)
#=================================================================================================================================
    #Delete the PanSTARRS image and directory
    #Comment this section out if you prefer to save the PanSTARRS image
    for i in range(len(all_field_files)):
        os.remove(all_field_files[i])
    if y_center < 0:
        os.rmdir(str(x_center)+'m'+str(y_center)[1:]+'/')
    if y_center > 0:
        os.rmdir(str(x_center)+'p'+str(y_center)+'/')



from .astrometrycorrection import *
from .create_err_plot import *
from .find_shifts import *
from .LRIS_Mask_Coords_to_WCS import *
from .precessionroutine import *
from .refractioncorrection import *
from .write_ds9_file import *



#autoslit precession routine

import numpy as np
from astropy.table import Table,Column
from astropy.io import ascii

#Taken from Autoslit and adapted for python
def precession(ra,dec,EPOCH,NEW_EPOCH):
    PI     = 3.14159265358979324
    RPD = PI/180.
    DPR = 1/RPD
    HALFPI = PI/2.
    TWOPI = 2.*PI
    DDEC = 0.0
    DRA = 0.0

    RA = ra*RPD
    DEC = dec*RPD
    DYEAR = NEW_EPOCH - EPOCH
    DELTADEC = DYEAR * DDEC
    DELTADEC = DELTADEC * RPD / 3600.
    DEC = DEC + DELTADEC

  #     Proper motion: RA is in radians; DRA is in sec of time/year.
    DELTARA = DYEAR * DRA
    DELTARA = DELTARA * 15. * RPD / 3600.
    RA = RA + DELTARA
  #     Calculate time parameters
    DT  = (EPOCH - NEW_EPOCH) * .001
    TAU = (EPOCH) * 0.001 - 1.900

    Z0 = (0.1117133 + 6.77527E-4 * TAU) * DT +     (1.4656E-4 - 1.31E-6    * TAU) * DT*DT + 8.726E-5 * DT**3

    Z = Z0 + (3.843118E-4 + 3.1998E-6 * TAU)*DT*DT + 1.5514E-6*DT**3

    ZJ = (0.097189874 - 4.13692E-4*TAU - 1.79381E-6*TAU*TAU) * DT -   (2.0687E-4 + 1.79381E-6*TAU) * DT*DT - 2.02652E-4 * DT**3

    Z = HALFPI + Z
    Z0 = Z0 - HALFPI
    ZJ = -ZJ
    XOLD = np.zeros(3)
    X1 = np.zeros(3)
    X2 = np.zeros(3)
  #     Precess coordinates using ROTATE().
    XOLD[0] = np.cos(RA) * np.cos(DEC)
    XOLD[1] = np.sin(RA) * np.cos(DEC)
    XOLD[2] = np.sin(DEC)
    X2 = ROTATE(2,Z,XOLD)
    X1 = ROTATE(0,ZJ,X2)
    X2 = ROTATE(2,Z0,X1)

    NEW_DEC = np.arcsin(X2[2])
    NEW_RA  = np.arccos(X2[0]/np.cos(NEW_DEC))

    if (X2[1] < 0.0):
        NEW_RA = TWOPI - NEW_RA
    RA = NEW_RA*DPR
    DEC = NEW_DEC*DPR

    return([RA,DEC])
#=================================================================================================================================
def ROTATE(M,ALPHA,XOLD):
    S = np.zeros(shape=(3,3))

    for i in range(0,3):
         for j in range(0,3):
            if ((i != M) and (j != M)):
               if (i != j):
                  if (i > j):
                      S[i,j] = (-1.)*np.sin(ALPHA)
                  if (i < j):
                      S[i,j] = np.sin(ALPHA)
               else:
                  S[i,j] = np.cos(ALPHA)
            else:
               S[i,j] = 0.0
               if (i == j):
                   S[i,j]=1.0

#     Rotate vector.
    XNEW = TRNSFM(S,XOLD)
    return(XNEW)

def TRNSFM(S,XOLD):
  XNEW = np.zeros(3)
  for i in range(0,3):
     XNEW[i]=0.0


#     Multiply matrix by vector.
  for i in range(0,3):
     for j in range(0,3):
        XNEW[i] = XNEW[i] + S[i,j]*XOLD[j]
  return(XNEW)



#function for creating the ds9 region files

#read in packages
import numpy as np
from astropy.coordinates import SkyCoord, ICRS, Galactic, FK4, FK5
from astropy.table import Table,Column
from regions import DS9Parser, Regions

def create_ds9_file(data,ra_centers,dec_centers,theta,catalog_object_ra,catalog_object_dec,output_file):
#create a region file associated with the coordinates given in the data file
    str_data = Table()
    ra_str = Column(data['RA_Center'], name='RA_Center', dtype=str)
    dec_str = Column(data['Dec_Center'], name='Dec_Center', dtype=str)
    str_data.add_column(ra_str)
    str_data.add_column(dec_str)
    pa = str(theta)
#=================================================================================================================================
#calculate the height and width (or grab them from the data?)
    width1 = str(np.abs(np.max(data['X'][0:4])-np.min(data['X'][0:4]))/3600)
    height1 = str(np.abs(np.max(data['Y'][0:4])-np.min(data['Y'][0:4]))/3600)
#=================================================================================================================================
#write out boxes to the reg_string
    reg_list ='icrs\nbox('+str_data['RA_Center'][0]+','+str_data['Dec_Center'][0]+','+width1+','+height1+','+pa+') # color=green\n'

    i=4
    while i in range(1,int(len(str_data))):#-4):
        width = str(np.abs(np.max(data['X'][i:i+4])-np.min(data['X'][i:i+4]))/(0.7253 *0.99857*3600))
        height = str(np.abs(np.max(data['Y'][i:i+4])-np.min(data['Y'][i:i+4]))/(0.7253 *0.99857*3600))
        reg_string='nbox('+str_data['RA_Center'][i]+','+str_data['Dec_Center'][i]+','+width+','+height+','+pa+') # color=green\n'
        reg_list = reg_list + reg_string
        i = i+4
#=================================================================================================================================

    #combine the two sections so the slits and vectors are both included
    reg_list = Regions.parse(reg_list,format='ds9')
    reg_filename = output_file+'.reg'
    reg_list.write(reg_filename,format='ds9',overwrite=True)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['testreverseautoslitcode']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5',
 'astropy>=4.3.post1,<5.0',
 'astroquery>=0.4.3,<0.5.0',
 'matplotlib>=3.4.2,<4.0.0',
 'numpy>=1.21.1,<2.0.0',
 'panstamps>=0.6.1,<0.7.0',
 'pytest-shutil>=1.7.0,<2.0.0',
 'regions>=0.5,<0.6',
 'shapely']

setup_kwargs = {
    'name': 'testreverseautoslitcode',
    'version': '0.1.31',
    'description': 'Test Compiling of Code',
    'long_description': None,
    'author': 'Jessica Sullivan',
    'author_email': 'jsulli27@nd.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#main routine to calculate WCS positions from LRIS mask positions
#reads in file containing

#import packages
import sys
import numpy as np
from astropy.coordinates import SkyCoord, ICRS, Galactic, FK4, FK5
from astropy.table import Table,Column
from astropy.io import ascii,fits
from astropy.time import Time
import astropy.units as u
from testreverseautoslitcode import find_shifts as fs
from testreverseautoslitcode import refractioncorrection as ref
from testreverseautoslitcode import astrometrycorrection as ac
from testreverseautoslitcode import create_err_plot
from testreverseautoslitcode import write_ds9_file as w9f
from testreverseautoslitcode import precessionroutine as pr
from shutil import copyfile

def xytowcs(data_input_name,output_file):

    #set up some constants
    rpd = np.pi/180. #radians per degree
    rph = rpd*15     #radians per hour
    rpas = rpd/3600. #radians per arcsec
    mask_angle = 8.06 * rpd #angle of mask to the focal plane
    bend = 1.94 * rpd #angle of the bend in the mask
    x_center = 305. #point treated as the center on the ccd
  #  catalog_keyword = 'panstarrs'   #uncomment the desired catalog to be used
    catalog_keyword = 'gaia'
 #   catalog_keyword = 'custom'
    if catalog_keyword == 'custom':
        catalog_file = input('Catalog filename:')
    else:
        catalog_file = ''

#================================================================================================================================

#read in the data\
#copy the original file to a new file that will be the one to which the results are added
    copyfile(data_input_name, output_file+'.fits')
#Open the copied file
    hdu=fits.open(output_file+'.fits',mode='update')
    maskbluID = str(hdu[5].data['BluID'][0])

#obtain the rotation angle of the instrument

    rot_angle = hdu[2].data['PA_PNT'][0]

    print('Rotation Angle is selected at:',rot_angle)

#get the reference ra and dec
#assume this to be the center of the image

    ra0,dec0,equ = hdu[2].data['RA_PNT'][0], hdu[2].data['DEC_PNT'][0],str(hdu[2].data['EQUINPNT'][0])
    epoch = hdu[2].data['EQUINPNT'][0]

#================================================================================================================================
    #Correct the center of the mask for refraction
    #Taken directly from autoslit
    HA = 0   #hour angle
    wave = 6000./10000  #wavelength in microns
    lat = 19.828 #keck latitude
    temperature = 0
    pres = 486.  #atmos pressure
    ra0 = ra0*rpd
    dec0 = dec0*rpd
    H = HA*rph + ra0 - ra0
    cosz = np.sin(lat*rpd)*np.sin(dec0)+np.cos(lat*rpd)*np.cos(dec0)*np.cos(H)
    sinz = np.sqrt(1-(cosz**2))
    tanz = sinz/cosz
    if sinz != 0:
        sinQ = np.cos(lat*rpd)*np.sin(H)/sinz
        cosQ = (np.cos(np.pi/2-lat*rpd)-cosz*np.cos(np.pi/2-dec0))/(sinz*np.sin(np.pi/2-dec0))
    else:
        sinQ = 0
        cosQ = 0
    WAVERS = 1. / (wave * wave)
    N = 64.328 + (29498.1/(146.-WAVERS)) + (255.4/(41.-WAVERS))
    N = 1.*10**-6 * N
    TCORR = 1. + 0.003661 * temperature
    NUM = 720.88 * TCORR
    N = N * ((pres * (1.+(1.049-0.0157*temperature) * 1.E-6 * pres)) / NUM)

    R = N * 206265. * tanz

    DA1 = R * sinQ * rpas / np.cos(dec0)
    DD1 = R * cosQ * rpas

    #set the center of the mask to the corrected value
    ra0 =  ra0 +  DA1
    dec0 = dec0 + DD1
    ra0 = str(ra0/rpd)
    dec0 = str(dec0/rpd)

    #Grab the date the mask was made
    creation_date = hdu[2].data['DesDate'][0]

    #Set the mask scale. This depends on whether the mask was made to be used with or without the ADC (ADC installed for B semester of 2007)
    if Time(creation_date) > Time('2007-08-01'):
        scale = 0.7253 *0.99857#scale of mask in mm/arcsec, corrected for ADC in use
        adcuse = 'post'
        print('--------------ADC Correction Used-----------------')
    else:
        scale = 0.7253 #scale of mask in mm/arcsec, not corrected for ADC in use
        print('--------------NO ADC Correction Used-----------------')
        adcuse = 'pre'
    x0 = x_center / scale

    ref_system = str(np.chararray.lower(str(hdu[2].data['RADEPNT'][0])))

#convert reference ra,dec to decimal degrees
#correct for precession of coordiantes based on the equinox they are given in

#set up SkyCoord object for the reference RA,Dec
    temp = SkyCoord(ra0+' '+dec0,frame=ref_system,unit=(u.deg,u.deg),equinox='J'+equ)
    ra0 = temp.ra.deg
    dec0 = temp.dec.deg
    racenter = temp.ra.deg
    deccenter = temp.dec.deg
    print('Reference Coordinates set at: (',ra0,',',dec0,')')

#read in the slit data
    print('------------Reading in Slit Data-----------------')
    data = Table()
    x1col = Column(hdu[6].data['slitX1'][0:1], name='slitX1', dtype=float)
    y1col = Column(hdu[6].data['slitY1'][0:1], name='slitY1', dtype=float)
    data.add_column(x1col)
    data.add_column(y1col)
    data.add_row([hdu[6].data['slitX2'][0],hdu[6].data['slitY2'][0]])
    data.add_row([hdu[6].data['slitX3'][0],hdu[6].data['slitY3'][0]])
    data.add_row([hdu[6].data['slitX4'][0],hdu[6].data['slitY4'][0]])

    for i in range(1,len(hdu[6].data['slitX1'])):
        data.add_row([hdu[6].data['slitX1'][i],hdu[6].data['slitY1'][i]])
        data.add_row([hdu[6].data['slitX2'][i],hdu[6].data['slitY2'][i]])
        data.add_row([hdu[6].data['slitX3'][i],hdu[6].data['slitY3'][i]])
        data.add_row([hdu[6].data['slitX4'][i],hdu[6].data['slitY4'][i]])

    data.rename_column('slitX1','X')
    data.rename_column('slitY1','Y')

    for i in range(len(data['X'])):
        data['X'][i] = np.float(data['X'][i])
        data['Y'][i] = np.float(data['Y'][i])

#The data from the html mask files is in the milling machine coordinate system. We have to convert to the mask coordinate
#system before working with the data
#x_mask = y_mill + 172.7
#y+mask = -x_mill +177.8

    x_mill = data['X']
    y_mill = data['Y']


    for i in range(len(data['X'])):
        a = y_mill[i]+172.7
        b = -x_mill[i]+177.8
        data['X'][i] = a
        data['Y'][i] = b

    x_input = data['X']
    y_input = data['Y']

    #apply the astrometry correction to the x,y values in the mask frame
    print('-------------------Applying Distortion Correction-------------------')
    astro_table = ac.astrometry_calc(ra0,dec0)
    for i in range(len(data['X'])):
        distance = np.zeros(len(astro_table))
        for j in range(len(astro_table)):
            distance[j] = np.sqrt(((data['X'][i]-astro_table['Aparent X'][j]))**2+(data['Y'][i]-astro_table['Aparent Y'][j])**2)
        val = np.argmin(distance)
        data['X'][i]= data['X'][i]-astro_table['X Offset'][val]
        data['Y'][i]= data['Y'][i]-astro_table['Y Offset'][val]


#add columns to data table to eventually hold the calculated RA and Dec for each object
    data.add_column(0.00000000000,name='Calc_RA')
    data.add_column(0.00000000000,name='Calc_Dec')

#=====================================================================================================================================

#calculate the RA,Dec from mask coordinates

#take theta to negative theta (internal angle opposite to recorded position angle)
    theta = (-rot_angle) * rpd

    print('--------------------Completing Inverse Gnomic Projection--------------------')
#have to correct the center coordiantes before inverse gnomic projectoin
    ra0 = ra0*rpd - (x0*np.cos(theta)*rpas/np.cos(dec0*rpd))
    dec0 = dec0*rpd - x0*np.sin(theta)*rpas

    for i in range(len(data['X'])):

#correct the x and y coords for the bend and tilt in the mask
        x_prime = np.cos(mask_angle)*(x_input[i])/scale
        y_prime = np.cos(bend)*(y_input[i])/scale

#take x,y to the eta and nu gnomic projection coordinates (rotation matrix)
        eta = (np.cos(theta)*x_prime-np.sin(theta)*y_prime)*rpas
        nu =  (np.sin(theta)*x_prime+np.cos(theta)*y_prime)*rpas
#take eta and nu to ra and dec (standard already calculated inversion)
        rho = np.sqrt(eta**2+nu**2)
        c = np.arctan2(rho,1.)

#calculate the final ra and dec using the inverse gnomic projection
        ra_t = eta*np.sin(c)
        ra_b = (rho*np.cos(dec0)*np.cos(c))-(nu*np.sin(dec0)*np.sin(c))
        dec_f = np.cos(c)*np.sin(dec0)
        dec_s = (nu*np.sin(c)*np.cos(dec0))/rho
        RA = (ra0+np.arctan2(ra_t/ra_b,1.))/rpd
        Dec = np.arcsin(dec_f+dec_s)/rpd

        data['Calc_RA'][i]=RA
        data['Calc_Dec'][i]=Dec

#===================================================================================================================================
    #Now apply refraction correction to each of the points on the mask
    print('-----------------Calculating Refraction Lookup Table------------------------')
    refraction_table = ref.refraction_calc(racenter*rpd,deccenter*rpd)
    #apply correction for each point
    for i in range(len(data['Calc_RA'])):
        distance = np.zeros(len(refraction_table))
        for j in range(len(refraction_table)):
            distance[j] = np.sqrt(((data['Calc_RA'][i]-refraction_table['Aparent RA'][j])*np.cos(data['Calc_Dec'][i]))**2+(data['Calc_Dec'][i]-refraction_table['Aparent Dec'][j])**2)
        val = np.argmin(distance)
        data['Calc_RA'][i]= data['Calc_RA'][i]-refraction_table['RA Offset'][val]
        data['Calc_Dec'][i]= data['Calc_Dec'][i]-refraction_table['Dec Offset'][val]

    for i in range(len(data['Calc_RA'])):
         temp1 = pr.precession(data['Calc_RA'][i],data['Calc_Dec'][i],epoch,2000.)
         data['Calc_RA'][i] = temp1[0]
         data['Calc_Dec'][i] = temp1[1]
#        temp = SkyCoord(str(data['Calc_RA'][i])+' '+str(data['Calc_Dec'][i]),frame=ref_system,unit=(u.deg,u.deg),equinox='J'+equ)
#        fk5_2000 = FK5(equinox='J2000')
    #take RA,Dec and transform to the J2000 frame
#        temp1=temp.transform_to(fk5_2000)
#        data['Calc_RA'][i] = temp1.ra.deg
#        data['Calc_Dec'][i] = temp1.dec.deg

#====================================================================================================================================
#calculate the average offset for the dataset and refraction correction
    print('----------------Calculating Mask Shift-----------------')
    data,x_centers,y_centers,ra_shifted_centers,dec_shifted_centers,catalog_obj_ra,catalog_obj_dec,objects_ra,objects_dec = fs.get_shift(data,theta,catalog_keyword,output_file,ref_system,racenter,deccenter,catalog_file,maskbluID,adcuse)

    #create the error plot
    print('-----------------Creating Quick Look Plot---------------')
    try:
        create_err_plot(data,catalog_obj_ra,catalog_obj_dec,objects_ra,objects_dec,output_file)
    except:
        print('ISSUE WITH QUICK LOOK PLOT...CHECK RESULTS')

    w9f.create_ds9_file(data,ra_shifted_centers,dec_shifted_centers,rot_angle,catalog_obj_ra,catalog_obj_dec,output_file)
#=====================================================================================================================================
#update the fits file extension to include the calculated center positions of the slits
    hdu[3].data['slitRA'] = data['RA_Center'][::4]
    hdu[3].data['slitDec'] = data['Dec_Center'][::4]
    hdu.flush()

#write out the data and results to the output file
    ascii.write(data,output_file+'.csv',format='csv',overwrite=True)



#read the file given in the command line and read

import sys
import numpy as np
from astropy.coordinates import SkyCoord, ICRS, Galactic, FK4, FK5
from astropy.table import Table,Column
from astropy.io import ascii,fits
from astropy.time import Time
from regions import DS9Parser, write_ds9
import astropy.units as u
from testreverseautoslitcode import find_shifts as fs
from testreverseautoslitcode import LRIS_Mask_Coords_to_WCS as lriscoords

'''
Read in the input file from the command line
Input file should have the mask files and desired output filename on individual lines separated by a space
'''

#Read in the file line by line
f = open(sys.argv[1],"r")
contents = f.readlines()
f.close()

#remove the "enter" from the end of lines if needed
for i in range(len(contents)):
    if contents[i][-1:] == '\n':
        contents[i] = contents[i][:-1]

#split the command into the mask file name and the desired output file name
    names = contents[i].split(' ')
    mask_file= ''
    if len(names) > 2:
        for j in range(len(names)-1):
            mask_file = mask_file +' '+ names[j]
        mask_file = mask_file[1:]
        output_file = names[len(names)-1]
    else:
        mask_file = names[0]
        output_file = names[1]

    print('Opening File:',mask_file)
    print('Output File Name:',output_file)
    lriscoords.xytowcs(mask_file,output_file)
